@@ -30,22 +30,33 @@
 #include "string.h"
 
 #if defined(HAVE_UNISTD_H)
+
 #include <unistd.h>
+
 #endif
+
 #include <fstream>
 #include <iostream>
 #include <sstream>
 #include <assert.h>
+#include <algorithm>
+#include <cctype>
 
 #include <libheif/heif.h>
 
 #include "encoder.h"
+
 #if HAVE_LIBJPEG
+
 #include "encoder_jpeg.h"
+
 #endif
 #if HAVE_LIBPNG
+
 #include "encoder_png.h"
+
 #endif
+
 #include "encoder_y4m.h"
 
 #if defined(_MSC_VER)
@@ -54,19 +65,24 @@
 
 #define UNUSED(x) (void)x
 
-static int usage(const char* command) {
-  fprintf(stderr, "USAGE: %s [-q quality] <filename> <output>\n", command);
+static int usage(const char* command)
+{
+  fprintf(stderr, "USAGE: %s [-q quality 0..100] <filename> <output>\n", command);
   return 1;
 }
 
-class ContextReleaser {
- public:
-  ContextReleaser(struct heif_context* ctx) : ctx_(ctx) {}
-  ~ContextReleaser() {
+class ContextReleaser
+{
+public:
+  ContextReleaser(struct heif_context* ctx) : ctx_(ctx)
+  {}
+
+  ~ContextReleaser()
+  {
     heif_context_free(ctx_);
   }
 
- private:
+private:
   struct heif_context* ctx_;
 };
 
@@ -77,11 +93,11 @@ int main(int argc, char** argv)
   UNUSED(quality);  // The quality will only be used by encoders that support it.
   while ((opt = getopt(argc, argv, "q:")) != -1) {
     switch (opt) {
-    case 'q':
-      quality = atoi(optarg);
-      break;
-    default: /* '?' */
-      return usage(argv[0]);
+      case 'q':
+        quality = atoi(optarg);
+        break;
+      default: /* '?' */
+        return usage(argv[0]);
     }
   }
 
@@ -94,34 +110,39 @@ int main(int argc, char** argv)
   std::string output_filename(argv[optind++]);
 
   std::unique_ptr<Encoder> encoder;
-  if (output_filename.size() > 4 &&
-      (output_filename.find(".jpg") == output_filename.size() - 4 ||
-       output_filename.find(".jpeg") == output_filename.size() - 5)) {
+
+  size_t dot_pos = output_filename.rfind('.');
+  if (dot_pos != std::string::npos) {
+    std::string suffix_lowercase = output_filename.substr(dot_pos + 1);
+
+    std::transform(suffix_lowercase.begin(), suffix_lowercase.end(),
+                   suffix_lowercase.begin(), ::tolower);
+
+    if (suffix_lowercase == "jpg" || suffix_lowercase == "jpeg") {
 #if HAVE_LIBJPEG
-    static const int kDefaultJpegQuality = 90;
-    if (quality == -1) {
-      quality = kDefaultJpegQuality;
-    }
-    encoder.reset(new JpegEncoder(quality));
+      static const int kDefaultJpegQuality = 90;
+      if (quality == -1) {
+        quality = kDefaultJpegQuality;
+      }
+      encoder.reset(new JpegEncoder(quality));
 #else
-    fprintf(stderr, "JPEG support has not been compiled in.\n");
-    return 1;
+      fprintf(stderr, "JPEG support has not been compiled in.\n");
+      return 1;
 #endif  // HAVE_LIBJPEG
-  }
+    }
 
-  if (output_filename.size() > 4 &&
-      output_filename.find(".png") == output_filename.size() - 4) {
+    if (suffix_lowercase == "png") {
 #if HAVE_LIBPNG
-    encoder.reset(new PngEncoder());
+      encoder.reset(new PngEncoder());
 #else
-    fprintf(stderr, "PNG support has not been compiled in.\n");
-    return 1;
+      fprintf(stderr, "PNG support has not been compiled in.\n");
+      return 1;
 #endif  // HAVE_LIBPNG
-  }
+    }
 
-  if (output_filename.size() > 4 &&
-      output_filename.find(".y4m") == output_filename.size() - 4) {
-    encoder.reset(new Y4MEncoder());
+    if (suffix_lowercase == "y4m") {
+      encoder.reset(new Y4MEncoder());
+    }
   }
 
   if (!encoder) {
@@ -138,15 +159,15 @@ int main(int argc, char** argv)
 
   std::ifstream istr(input_filename.c_str(), std::ios_base::binary);
   uint8_t magic[12];
-  istr.read((char*)magic,12);
-  enum heif_filetype_result filetype_check = heif_check_filetype(magic,12);
+  istr.read((char*) magic, 12);
+  enum heif_filetype_result filetype_check = heif_check_filetype(magic, 12);
   if (filetype_check == heif_filetype_no) {
-    fprintf(stderr, "Input file is not an HEIF file\n");
+    fprintf(stderr, "Input file is not an HEIF/AVIF file\n");
     return 1;
   }
 
   if (filetype_check == heif_filetype_yes_unsupported) {
-    fprintf(stderr, "Input file is an unsupported HEIF file type\n");
+    fprintf(stderr, "Input file is an unsupported HEIF/AVIF file type\n");
     return 1;
   }
 
@@ -156,7 +177,7 @@ int main(int argc, char** argv)
 
   struct heif_context* ctx = heif_context_alloc();
   if (!ctx) {
-    fprintf(stderr, "Could not create HEIF context\n");
+    fprintf(stderr, "Could not create context object\n");
     return 1;
   }
 
@@ -164,7 +185,7 @@ int main(int argc, char** argv)
   struct heif_error err;
   err = heif_context_read_from_file(ctx, input_filename.c_str(), nullptr);
   if (err.code != 0) {
-    std::cerr << "Could not read HEIF file: " << err.message << "\n";
+    std::cerr << "Could not read HEIF/AVIF file: " << err.message << "\n";
     return 1;
   }
 
@@ -176,7 +197,7 @@ int main(int argc, char** argv)
 
   printf("File contains %d images\n", num_images);
 
-  heif_item_id* image_IDs = (heif_item_id*)alloca(num_images * sizeof(heif_item_id));
+  heif_item_id* image_IDs = (heif_item_id*) alloca(num_images * sizeof(heif_item_id));
   num_images = heif_context_get_list_of_top_level_image_IDs(ctx, image_IDs, num_images);
 
 
@@ -185,21 +206,22 @@ int main(int argc, char** argv)
 
   for (int idx = 0; idx < num_images; ++idx) {
 
-    if (num_images>1) {
+    if (num_images > 1) {
       std::ostringstream s;
       s << output_filename.substr(0, output_filename.find_last_of('.'));
       s << "-" << image_index;
       s << output_filename.substr(output_filename.find_last_of('.'));
       filename.assign(s.str());
-    } else {
+    }
+    else {
       filename.assign(output_filename);
     }
 
     struct heif_image_handle* handle;
     err = heif_context_get_image_handle(ctx, image_IDs[idx], &handle);
     if (err.code) {
-      std::cerr << "Could not read HEIF image " << idx << ": "
-          << err.message << "\n";
+      std::cerr << "Could not read HEIF/AVIF image " << idx << ": "
+                << err.message << "\n";
       return 1;
     }
 
@@ -224,16 +246,17 @@ int main(int argc, char** argv)
     heif_decoding_options_free(decode_options);
     if (err.code) {
       heif_image_handle_release(handle);
-      std::cerr << "Could not decode HEIF image: " << idx << ": "
-          << err.message << "\n";
+      std::cerr << "Could not decode image: " << idx << ": "
+                << err.message << "\n";
       return 1;
     }
 
     if (image) {
       bool written = encoder->Encode(handle, image, filename.c_str());
       if (!written) {
-        fprintf(stderr,"could not write image\n");
-      } else {
+        fprintf(stderr, "could not write image\n");
+      }
+      else {
         printf("Written to %s\n", filename.c_str());
       }
       heif_image_release(image);
@@ -243,8 +266,8 @@ int main(int argc, char** argv)
       if (has_depth) {
         heif_item_id depth_id;
         int nDepthImages = heif_image_handle_get_list_of_depth_image_IDs(handle, &depth_id, 1);
-        assert(nDepthImages==1);
-        (void)nDepthImages;
+        assert(nDepthImages == 1);
+        (void) nDepthImages;
 
         struct heif_image_handle* depth_handle;
         err = heif_image_handle_get_depth_image_handle(handle, depth_id, &depth_handle);
@@ -276,8 +299,9 @@ int main(int argc, char** argv)
 
         written = encoder->Encode(depth_handle, depth_image, s.str());
         if (!written) {
-          fprintf(stderr,"could not write depth image\n");
-        } else {
+          fprintf(stderr, "could not write depth image\n");
+        }
+        else {
           printf("Depth image written to %s\n", s.str().c_str());
         }
       }
