@@ -29,8 +29,11 @@
 
 #include <errno.h>
 #include <string.h>
+
 #if defined(HAVE_UNISTD_H)
+
 #include <unistd.h>
+
 #else
 #define STDOUT_FILENO 1
 #endif
@@ -45,37 +48,37 @@
 
 
 static struct option long_options[] = {
-  //{"write-raw", required_argument, 0, 'w' },
-  //{"output",    required_argument, 0, 'o' },
-  {"decode-img", required_argument,      0, 'd' },
-  {"metadata",   required_argument,      0, 'm' },
-  {0,         0,                 0,  0 }
+    //{"write-raw", required_argument, 0, 'w' },
+    //{"output",    required_argument, 0, 'o' },
+    {"decode-img", required_argument, 0, 'd'},
+    {"metadata",   required_argument, 0, 'm'},
+    {0, 0,                            0, 0}
 };
 
 void show_help(const char* argv0)
 {
-    fprintf(stderr," heif-test  libheif version: %s\n",heif_get_version());
-    fprintf(stderr,"------------------------------------\n");
-    fprintf(stderr,"usage: heif-test [options] image.heic\n");
-    fprintf(stderr,"\n");
-    fprintf(stderr,"options:\n");
-    fprintf(stderr,"  -d, --decode-img ID  decode image and output raw pixel data of all planes\n");
-    fprintf(stderr,"  -m, --metadata ID    output metadata\n");
-    fprintf(stderr,"  -h, --help           show help\n");
+  fprintf(stderr, " heif-test  libheif version: %s\n", heif_get_version());
+  fprintf(stderr, "------------------------------------\n");
+  fprintf(stderr, "usage: heif-test [options] image.heic\n");
+  fprintf(stderr, "\n");
+  fprintf(stderr, "options:\n");
+  fprintf(stderr, "  -d, --decode-img ID  decode image and output raw pixel data of all planes\n");
+  fprintf(stderr, "  -m, --metadata ID    output metadata\n");
+  fprintf(stderr, "  -h, --help           show help\n");
 }
 
 
 std::pair<heif_item_id, heif_item_id> parse_id_pair(std::string s)
 {
   std::string::size_type p = s.find_first_of(':');
-  if (p==std::string::npos) {
-    fprintf(stderr,"id pair has to be in this format: 'ID:ID'\n");
+  if (p == std::string::npos) {
+    fprintf(stderr, "id pair has to be in this format: 'ID:ID'\n");
     exit(1);
   }
 
   std::pair<heif_item_id, heif_item_id> pair;
-  pair.first  = atoi(s.substr(0,p).c_str());
-  pair.second = atoi(s.substr(p+1).c_str());
+  pair.first = atoi(s.substr(0, p).c_str());
+  pair.second = atoi(s.substr(p + 1).c_str());
   return pair;
 }
 
@@ -83,7 +86,7 @@ std::pair<heif_item_id, heif_item_id> parse_id_pair(std::string s)
 int main(int argc, char** argv)
 {
   std::vector<heif_item_id> image_IDs;
-  std::vector<std::pair<heif_item_id,heif_item_id>> metadata_IDs; // first: image, second: metadata
+  std::vector<std::pair<heif_item_id, heif_item_id>> metadata_IDs; // first: image, second: metadata
 
   while (true) {
     int option_index = 0;
@@ -92,19 +95,19 @@ int main(int argc, char** argv)
       break;
 
     switch (c) {
-    case 'd':
-      image_IDs.push_back(atoi(optarg));
-      break;
-    case 'm':
-      metadata_IDs.push_back(parse_id_pair(optarg));
-      break;
-    case 'h':
-      show_help(argv[0]);
-      return 0;
+      case 'd':
+        image_IDs.push_back(atoi(optarg));
+        break;
+      case 'm':
+        metadata_IDs.push_back(parse_id_pair(optarg));
+        break;
+      case 'h':
+        show_help(argv[0]);
+        return 0;
     }
   }
 
-  if (optind != argc-1) {
+  if (optind != argc - 1) {
     show_help(argv[0]);
     return 0;
   }
@@ -126,7 +129,7 @@ int main(int argc, char** argv)
 
       heif::Image img = handle.decode_image(heif_colorspace_undefined, heif_chroma_undefined);
 
-      std::vector<heif_channel> channel_candidates  {
+      std::vector<heif_channel> channel_candidates{
           heif_channel_Y,
           heif_channel_Cb,
           heif_channel_Cr,
@@ -135,18 +138,18 @@ int main(int argc, char** argv)
           heif_channel_B,
           heif_channel_Alpha,
           heif_channel_interleaved
-          };
+      };
 
       for (heif_channel channel : channel_candidates) {
         if (img.has_channel(channel)) {
-          int width  = img.get_width(channel);
+          int width = img.get_width(channel);
           int height = img.get_height(channel);
-          int bytes = (img.get_bits_per_pixel(channel)+7)/8;
+          int bytes = (img.get_bits_per_pixel(channel) + 7) / 8;
 
           int stride;
           const uint8_t* p = img.get_plane(channel, &stride);
-          for (int y=0;y<height;y++) {
-            fwrite(p+y*stride, width, bytes, stdout);
+          for (int y = 0; y < height; y++) {
+            fwrite(p + y * stride, width, bytes, stdout);
           }
         }
       }
@@ -158,10 +161,10 @@ int main(int argc, char** argv)
     for (auto idpair : metadata_IDs) {
       heif::ImageHandle handle = ctx.get_image_handle(idpair.first);
       std::vector<uint8_t> data = handle.get_metadata(idpair.second);
-      fwrite(data.data(), data.size(),1, stdout);
+      fwrite(data.data(), data.size(), 1, stdout);
     }
   }
-  catch (heif::Error err) {
+  catch (const heif::Error& err) {
     std::cerr << err.get_message() << "\n";
   }
 
