@@ -20,6 +20,8 @@
 
 #include "heif.h"
 #include "heif_plugin.h"
+#include "heif_colorconversion.h"
+#include "heif_api_structs.h"
 
 #if defined(HAVE_CONFIG_H)
 #include "config.h"
@@ -31,12 +33,14 @@
 #include <libde265/de265.h>
 #include <stdio.h>
 
+using namespace heif;
+
+
 struct libde265_decoder
 {
   de265_decoder_context* ctx;
 };
 
-static const char kSuccess[] = "Success";
 static const char kEmptyString[] = "";
 
 static const int LIBDE265_PLUGIN_PRIORITY = 100;
@@ -53,8 +57,8 @@ static const char* libde265_plugin_name()
   const char* libde265_version = de265_get_version();
 
   if (strlen(libde265_version) + 10 < MAX_PLUGIN_NAME_LENGTH) {
-    strcat(plugin_name,", version ");
-    strcat(plugin_name,libde265_version);
+    strcat(plugin_name, ", version ");
+    strcat(plugin_name, libde265_version);
   }
 
   return plugin_name;
@@ -85,54 +89,72 @@ static int libde265_does_support_format(enum heif_compression_format format)
 
 
 static struct heif_error convert_libde265_image_to_heif_image(struct libde265_decoder* decoder,
-                                                              const struct de265_image* de265img, struct heif_image** image)
+                                                              const struct de265_image* de265img,
+                                                              struct heif_image** image)
 {
-  struct heif_image* out_img;
-  struct heif_error err = heif_image_create(
-    de265_get_image_width(de265img, 0),
-    de265_get_image_height(de265img, 0),
-    heif_colorspace_YCbCr, // TODO
-    (heif_chroma)de265_get_chroma_format(de265img),
-    &out_img);
-  if (err.code != heif_error_Ok) {
-    return err;
-  }
+  bool is_mono = (de265_get_chroma_format(de265img) == de265_chroma_mono);
+
+  std::shared_ptr<HeifPixelImage> yuv_img = std::make_shared<HeifPixelImage>();
+  yuv_img->create(de265_get_image_width(de265img, 0),
+                  de265_get_image_height(de265img, 0),
+                  is_mono ? heif_colorspace_monochrome : heif_colorspace_YCbCr,
+                  (heif_chroma) de265_get_chroma_format(de265img));
 
   // --- transfer data from de265_image to HeifPixelImage
 
   heif_channel channel2plane[3] = {
-    heif_channel_Y,
-    heif_channel_Cb,
-    heif_channel_Cr
+      heif_channel_Y,
+      heif_channel_Cb,
+      heif_channel_Cr
   };
 
 
-  for (int c=0;c<3;c++) {
-    int bpp = de265_get_bits_per_pixel(de265img, c);
+  int bpp = de265_get_bits_per_pixel(de265img, 0);
+
+  int nPlanes = (is_mono ? 1 : 3);
+
+  for (int c = 0; c < nPlanes; c++) {
+    if (de265_get_bits_per_pixel(de265img, c) != bpp) {
+      struct heif_error err = {heif_error_Unsupported_feature,
+                               heif_suberror_Unsupported_color_conversion,
+                               "Channels with different number of bits per pixel are not supported"};
+      return err;
+    }
 
     int stride;
     const uint8_t* data = de265_get_image_plane(de265img, c, &stride);
 
     int w = de265_get_image_width(de265img, c);
     int h = de265_get_image_height(de265img, c);
+    if (w <= 0 || h <= 0) {
+      struct heif_error err = {heif_error_Decoder_plugin_error,
+                               heif_suberror_Invalid_image_size,
+                               kEmptyString};
+      return err;
+    }
 
-    err = heif_image_add_plane(out_img, channel2plane[c], w,h, bpp);
-    if (err.code != heif_error_Ok) {
-      heif_image_release(out_img);
+    if (!yuv_img->add_plane(channel2plane[c], w, h, bpp)) {
+      struct heif_error err = {heif_error_Memory_allocation_error,
+                               heif_suberror_Unspecified,
+                               "Cannot allocate memory for image plane"};
       return err;
     }
 
     int dst_stride;
-    uint8_t* dst_mem = heif_image_get_plane(out_img, channel2plane[c], &dst_stride);
+    uint8_t* dst_mem = yuv_img->get_plane(channel2plane[c], &dst_stride);
 
-    int bytes_per_pixel = (bpp+7)/8;
+    int bytes_per_pixel = (bpp + 7) / 8;
 
-    for (int y=0;y<h;y++) {
-      memcpy(dst_mem + y*dst_stride, data + y*stride, w * bytes_per_pixel);
+    for (int y = 0; y < h; y++) {
+      memcpy(dst_mem + y * dst_stride, data + y * stride, w * bytes_per_pixel);
     }
   }
 
-  *image = out_img;
+
+  *image = new heif_image;
+  (*image)->image = yuv_img;
+
+  struct heif_error err = {heif_error_Ok, heif_suberror_Unspecified, kSuccess};
   return err;
 }
 
@@ -140,7 +162,7 @@ static struct heif_error convert_libde265_image_to_heif_image(struct libde265_de
 static struct heif_error libde265_new_decoder(void** dec)
 {
   struct libde265_decoder* decoder = new libde265_decoder();
-  struct heif_error err = { heif_error_Ok, heif_suberror_Unspecified, kSuccess };
+  struct heif_error err = {heif_error_Ok, heif_suberror_Unspecified, kSuccess};
 
   decoder->ctx = de265_new_decoder();
 #if defined(__EMSCRIPTEN__)
@@ -158,10 +180,10 @@ static struct heif_error libde265_new_decoder(void** dec)
 
 static void libde265_free_decoder(void* decoder_raw)
 {
-  struct libde265_decoder* decoder = (struct libde265_decoder*)decoder_raw;
+  struct libde265_decoder* decoder = (struct libde265_decoder*) decoder_raw;
 
   de265_error err = de265_free_decoder(decoder->ctx);
-  (void)err;
+  (void) err;
 
   delete decoder;
 }
@@ -177,7 +199,7 @@ static struct heif_error libde265_v2_push_data(void* decoder_raw, const void* da
 
   size_t ptr=0;
   while (ptr < size) {
-    if (ptr+4 > size) {
+    if (4 > size - ptr) {
       struct heif_error err = { heif_error_Decoder_plugin_error,
                                 heif_suberror_End_of_data,
                                 kEmptyString };
@@ -189,7 +211,7 @@ static struct heif_error libde265_v2_push_data(void* decoder_raw, const void* da
     uint32_t nal_size = (cdata[ptr]<<24) | (cdata[ptr+1]<<16) | (cdata[ptr+2]<<8) | (cdata[ptr+3]);
     ptr+=4;
 
-    if (ptr+nal_size > size) {
+    if (nal_size > size - ptr) {
       //sstr << "NAL size (" << size32 << ") exceeds available data in file ("
       //<< data_bytes_left_to_read << ")";
 
@@ -209,7 +231,8 @@ static struct heif_error libde265_v2_push_data(void* decoder_raw, const void* da
 }
 
 
-static struct heif_error libde265_v2_decode_image(void* decoder_raw, struct heif_image** out_img)
+static struct heif_error libde265_v2_decode_image(void* decoder_raw,
+                                                  struct heif_image** out_img)
 {
   struct libde265_decoder* decoder = (struct libde265_decoder*)decoder_raw;
 
@@ -221,7 +244,8 @@ static struct heif_error libde265_v2_decode_image(void* decoder_raw, struct heif
   if (action==de265_action_get_image) {
     const de265_image* img = de265_get_next_picture(decoder->ctx);
     if (img) {
-      struct heif_error err = convert_libde265_image_to_heif_image(decoder, img, out_img);
+      struct heif_error err = convert_libde265_image_to_heif_image(decoder, img,
+                                                                   out_img);
       de265_release_picture(img);
 
       return err;
@@ -236,51 +260,53 @@ static struct heif_error libde265_v2_decode_image(void* decoder_raw, struct heif
 
 static struct heif_error libde265_v1_push_data(void* decoder_raw, const void* data, size_t size)
 {
-  struct libde265_decoder* decoder = (struct libde265_decoder*)decoder_raw;
+  struct libde265_decoder* decoder = (struct libde265_decoder*) decoder_raw;
 
-  const uint8_t* cdata = (const uint8_t*)data;
+  const uint8_t* cdata = (const uint8_t*) data;
 
-  size_t ptr=0;
+  size_t ptr = 0;
   while (ptr < size) {
-    if (ptr+4 > size) {
-      struct heif_error err = { heif_error_Decoder_plugin_error,
-                                heif_suberror_End_of_data,
-                                kEmptyString };
+    if (4 > size - ptr) {
+      struct heif_error err = {heif_error_Decoder_plugin_error,
+                               heif_suberror_End_of_data,
+                               kEmptyString};
       return err;
     }
 
-    uint32_t nal_size = (cdata[ptr]<<24) | (cdata[ptr+1]<<16) | (cdata[ptr+2]<<8) | (cdata[ptr+3]);
-    ptr+=4;
+    uint32_t nal_size = (cdata[ptr] << 24) | (cdata[ptr + 1] << 16) | (cdata[ptr + 2] << 8) | (cdata[ptr + 3]);
+    ptr += 4;
 
-    if (ptr+nal_size > size) {
-      struct heif_error err = { heif_error_Decoder_plugin_error,
-                                heif_suberror_End_of_data,
-                                kEmptyString };
+    if (nal_size > size - ptr) {
+      struct heif_error err = {heif_error_Decoder_plugin_error,
+                               heif_suberror_End_of_data,
+                               kEmptyString};
       return err;
     }
 
-    de265_push_NAL(decoder->ctx, cdata+ptr, nal_size, 0, nullptr);
+    de265_push_NAL(decoder->ctx, cdata + ptr, nal_size, 0, nullptr);
     ptr += nal_size;
   }
 
   // TODO(farindk): Set "err" if data could not be pushed
   //de265_push_data(decoder->ctx, data, size, 0, nullptr);
 
-  struct heif_error err = { heif_error_Ok, heif_suberror_Unspecified, kSuccess };
+  struct heif_error err = {heif_error_Ok, heif_suberror_Unspecified, kSuccess};
   return err;
 }
 
 
-static struct heif_error libde265_v1_decode_image(void* decoder_raw, struct heif_image** out_img)
+static struct heif_error libde265_v1_decode_image(void* decoder_raw,
+                                                  struct heif_image** out_img)
 {
-  struct libde265_decoder* decoder = (struct libde265_decoder*)decoder_raw;
-  struct heif_error err = { heif_error_Ok, heif_suberror_Unspecified, kSuccess };
+  struct libde265_decoder* decoder = (struct libde265_decoder*) decoder_raw;
+  struct heif_error err = {heif_error_Ok, heif_suberror_Unspecified, kSuccess};
 
   de265_flush_data(decoder->ctx);
 
   // TODO(farindk): Set "err" if no image was decoded.
   int more;
   de265_error decode_err;
+  *out_img = nullptr;
   do {
     more = 0;
     decode_err = de265_decode(decoder->ctx, &more);
@@ -291,6 +317,10 @@ static struct heif_error libde265_v1_decode_image(void* decoder_raw, struct heif
 
     const struct de265_image* image = de265_get_next_picture(decoder->ctx);
     if (image) {
+      // TODO(farindk): Should we return the first image instead?
+      if (*out_img) {
+        heif_image_release(*out_img);
+      }
       err = convert_libde265_image_to_heif_image(decoder, image, out_img);
 
       de265_release_next_picture(decoder->ctx);
@@ -300,12 +330,10 @@ static struct heif_error libde265_v1_decode_image(void* decoder_raw, struct heif
   return err;
 }
 
+
 #endif
 
 
-
-
-
 #if LIBDE265_NUMERIC_VERSION >= 0x02000000
 
 static const struct heif_decoder_plugin decoder_libde265
@@ -324,20 +352,21 @@ static const struct heif_decoder_plugin decoder_libde265
 #else
 
 static const struct heif_decoder_plugin decoder_libde265
-{
-  1,
-  libde265_plugin_name,
-  libde265_init_plugin,
-  libde265_deinit_plugin,
-  libde265_does_support_format,
-  libde265_new_decoder,
-  libde265_free_decoder,
-  libde265_v1_push_data,
-  libde265_v1_decode_image
-};
+    {
+        1,
+        libde265_plugin_name,
+        libde265_init_plugin,
+        libde265_deinit_plugin,
+        libde265_does_support_format,
+        libde265_new_decoder,
+        libde265_free_decoder,
+        libde265_v1_push_data,
+        libde265_v1_decode_image
+    };
 
 #endif
 
-const struct heif_decoder_plugin* get_decoder_plugin_libde265() {
+const struct heif_decoder_plugin* get_decoder_plugin_libde265()
+{
   return &decoder_libde265;
 }
